; pulse_measure_decimal_standalone_505.a51:  displaying pulse width count in decimal (software count) 5/05
; CHANGE DISPLAY FOR STANDALONE: instead of @DPTR, write MSD to PORT2
;  for main computer, writes 16-bits to display, at 8000h, 80001h

; FOR STANDALONE USE, COMMENT OUT DPTR DISPLAY, UNCOMMENT WRITE TO P2
; FOR TESTING IN LAB MICRO, KILL WRITE TO P2, KEEP DPTR DISPLAY

; starts incrementing on fall of INT0, stops  on fall of INT1 (=rise of pulse), displays resulting 
;  timer count on displays, lights overflow LED on P1.1 in case of 16-bit roll-over

; rev path 3/16
	$NOSYMBOLS					; keeps listing short
	$INCLUDE (C:\SiLabs\MCU\INC\c8051f410.inc) 
	$INCLUDE (C:\SiLabs\MCU\INC\VECTORS320.INC)	; Tom's vectors definition file
	
	
	 COUNTFLAG EQU 0	; software flag that indicates edge has fallen 
	 STOPFLAG EQU 1	; ...indicates edge has risen
	 
	 OVERFLOW EQU TF2	; rollover flag for timer 2
	 
	 CLEARVAL SET 00h	; this is used to clear counts
 
	 CKMOD EQU 96h		; address of CKMOD register (a byte)

	 STACKBOT EQU 80h	; scratch RAM, OK place to put stack



	ORG 0H

	JMP START

	ORG 40h

START: MOV SP, #STACKBOT-1
	ACALL INITS 

	; ----NOW ENABLE INTERRUPTS----
	SETB IT0		; make INT0 Edge-sensitive	(p. 22)
	SETB EX0		; ...and enable INT0

	SETB IT1		; ...and INT1 edge too
	SETB EX1		; ...and enable INT1

	CLR IE0		; clear int request flags, in case they were pending
	CLR IE1
	
	SETB EA		; Global int enable        (pp.31-32)


	
REINIT: ACALL REINITS						; set up next pass
COUNTWHILE:	JNB COUNTFLAG, COUNTWHILE	; AWAIT falling edge of pulse--flag indicates falling edge
COUNTING: ACALL COUNTUP								; go count up
	JNB STOPFLAG, COUNTING				; carry on, till end of pulse stops counter
	CLR STOPFLAG	
	ACALL SHOWTIME								; on Stop, go show pulse width
	CLR IE0		; clear int request flags, in case they were pending
	CLR IE1

	SJMP REINIT									; await fresh pulse
;QUIT: SJMP QUIT	


; SUBROUTINES*******************************************
INITS: 	CLR STOPFLAG
	MOV DPTR, #8000h				; point to display (this doesn't change, thoughout
							;  pgm, except briefly to show 16-bit result
	CLR A
	MOVX @DPTR,A				; clean start										
	SETB P1.0					; turn off Overflow LED
	RET

REINITS: MOV R0,#0				; clear count
	MOV R1, #0
	CLR C
	CLR COUNTFLAG				; stop counting	
	CLR STOPFLAG
	SETB EX0					; enable falling-edge interrupts
	RET

;SHOWTIME: 	MOV P2, R1		; ...high byte
;	RET

; LAB COMPUTER VERSION: SHOW 16 BITS ON DISPLAYS, USING DPTR
SHOWTIME: MOV A, R0		; get count values to show on display	
	MOVX @DPTR,A		; this is low byte
	MOV A, R1
	INC DPTR
	MOVX @DPTR, A		; ...high byte
	DEC DPL			; restore pointer to original value
	RET
	

COUNTUP: PUSH ACC
	PUSH PSW
	MOV A, R0			; get low byte of count
	ADD A,#1				; increment count
	DA A					; decimal adjust
	MOV R0,A				; update running count, low byte
	MOV A,R1				; get hi byte of count
	ADDC A,#0			; increment hi byte only if there's a carry from low
	DA A
	MOV R1,A				; update runing count, hi byte
	JNC OKAY
	CLR P1.0				; turn on Overflow LED, if there's a carry OUT
OKAY: POP PSW
	POP ACC
	RET	


; ---ISR'S -------

	ORG INT0VECTOR
	CLR EX0				; disable further falling-edge interrupts
	SETB P1.0			; turn off Overflow LED	
	SETB COUNTFLAG		; start counting
	SETB EX1
	RETI
	
	ORG INT1VECTOR
	CLR EX1				; disable further rising-edge interrupts
	CLR COUNTFLAG		; stop counting
	SETB STOPFLAG
	RETI
	
	END
